package com.altinity.opsview.odw;

/**
 * Copyright (C) 2008 Opsera Limited. All rights reserved
 *
 * This file is part of Opsview
 * 
 * Opsview is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * Opsview is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Opsview; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 * 
 */

import java.io.File;

import org.dom4j.Document;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.jdbc.datasource.DriverManagerDataSource;

import com.altinity.opsview.odw.bc.AvailabilityReportBC;
import com.altinity.opsview.odw.bc.BaseReportBC;
import com.altinity.opsview.odw.bc.WeeklyServiceAvailabilityReportBC;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.exception.ReportException;
import com.excira.reports.ReportEngine;
import com.excira.utils.StringUtils;

public class ODWReportGenerator implements InitializingBean {

	// Objects injected by Spring
	private AvailabilityReportBC availBc;
	protected ODWReportConfigDAO configDao;
	private WeeklyServiceAvailabilityReportBC weeklyBc;

	/**
	 * 
	 */
	private void createDir(String dirName) {

		File dir = new File(dirName);
		if (!dir.exists())
			dir.mkdirs();
	}

	/**
	 * @throws Exception
	 * 
	 */
	public boolean generateReport(CommandLineParams params) throws Exception {
		
		System.out.println("Generating ODW3 Report...");
		
		try {
			
			if ( StringUtils.notNullOrEmpty(params.getUserName()) ) {
				((DriverManagerDataSource)configDao.getDataSource()).setUsername(params.getUserName());
				((DriverManagerDataSource)configDao.getDataSource()).setPassword(params.getPassword());
			}
			int reportTypeId = configDao.getReportTypeByReportId(new Integer(params.getReportId()));

			String currentDir = System.getProperty("user.dir");
			String xslFileName = getFileName(reportTypeId);
			String xslFilePath = currentDir + "/xsl/reports/" + xslFileName;
			String chartsRoot = currentDir + "/temp";
			String assetRoot = currentDir + "/assets";
			
			// Get the current directory
			String tempDir = currentDir + "/temp";

			// Make sure temp directories exist
			createDir(tempDir);			
			
			BaseReportBC bc = getBC(reportTypeId);
			
			// Setup bc
			if ( reportTypeId == 2 ) {
				int startOfWeek = configDao.getStartOfWeekByReportId(new Integer(params.getReportId()));
				params.setStartOfWeek(startOfWeek);
			}
			bc.init(params);
			bc.setChartsRoot(chartsRoot);
			
			// Test if the databases are alive
			// and give a reasonable error if they are not.
			bc.hitDBs();
			
			// Load report criteria
			// Returns true if data loaded for report period
			boolean dataLoaded = bc.loadReporCriteria();

			// Start building the report xml
			Document doc = bc.createNewReportDocument();
			
			// Basics and cover
			doc.getRootElement().addElement("charts-root").setText(chartsRoot);
			doc.getRootElement().addElement("asset-root").setText(assetRoot);
			doc.getRootElement().addElement("logo").setText("assets/opsview_436_100.gif");
			
			// availability summary
			// Only execute if data has been loaded for 
			// reporting period
			if ( dataLoaded )
				bc.createReport();
			
			// Add failure summary
			if ( !bc.getLog().isClean() )
					bc.createErrorsSummary();
		 
			// create report
			ReportEngine re = new ReportEngine();
	
			re.genratePDF(doc, xslFilePath, bc.getOutputFilePath());
			
			bc.cleanUp();
	
			System.out.println("Report generation complete...");
		}
		catch(ReportException e) {
			System.out.println(e.getMessage());
			return false;
		}
		
		return true;
	}

	/**
	 * 
	 */
	public void setAvailBc(AvailabilityReportBC bc) {
		this.availBc = bc;
	}
	

	/**
	 * 
	 */
	public void setWeeklyBc(WeeklyServiceAvailabilityReportBC bc) {
		this.weeklyBc = bc;
	}	
	
	/**
	 * 
	 */
	private BaseReportBC getBC(int reportTypeId) throws ReportException {
		
		if ( reportTypeId == 1 )
			return availBc;
		else if ( reportTypeId == 2 )
			return weeklyBc;
		else
			throw new ReportException("Unknown report type '" + reportTypeId + "' in getBC.");
	}
	
	/**
	 * 
	 */
	private String getFileName(int reportTypeId) throws ReportException {
		
		if ( reportTypeId == 1 )
			return "GenericAvailabilityReport.xsl";
		else if ( reportTypeId == 2 )
			return "WeeklyServiceAvailabilityReport.xsl";
		else
			throw new ReportException("Unknown report type '" + reportTypeId + "' in getFileName.");
	}

	/**
	 * 
	 */
	public void afterPropertiesSet() throws Exception {

	}

	/**
	 * 
	 */
	public void setConfigDao(ODWReportConfigDAO configDao) {
		this.configDao = configDao;
	}
}
